# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import BaseErrorResponse
from ..models import HTTPValidationError
from ..models import ListBaseResponseStr
from ..models import PagedResponseGetNamespaceEventResponse

from ..operations.telemetry import (
    GetEventsGameTelemetryV1AdminNamespacesNamespaceEventsGet,
)
from ..operations.telemetry import GetNamespacesGameTelemetryV1AdminNamespacesGet


@same_doc_as(GetEventsGameTelemetryV1AdminNamespacesNamespaceEventsGet)
def get_events_game_telemetry_v1_admin_namespaces_namespace_events_get(
    device_type: Optional[str] = None,
    end_time: Optional[str] = None,
    event_id: Optional[str] = None,
    event_name: Optional[str] = None,
    event_payload: Optional[str] = None,
    flight_id: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    start_time: Optional[str] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Events (get_events_game_telemetry_v1_admin_namespaces__namespace__events_get)

    This endpoint requires valid JWT token and telemetry permission
    This endpoint retrieves event list

    Properties:
        url: /game-telemetry/v1/admin/namespaces/{namespace}/events

        method: GET

        tags: ["telemetry"]

        consumes: []

        produces: ["application/json"]

        securities: [COOKIE_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        device_type: (deviceType) OPTIONAL str in query

        end_time: (endTime) OPTIONAL str in query

        event_id: (eventId) OPTIONAL str in query

        event_name: (eventName) OPTIONAL str in query

        event_payload: (eventPayload) OPTIONAL str in query

        flight_id: (flightId) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        start_time: (startTime) OPTIONAL str in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - PagedResponseGetNamespaceEventResponse (Successful Response)

        400: Bad Request - BaseErrorResponse (Bad Request)

        422: Unprocessable Entity - HTTPValidationError (Validation Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventsGameTelemetryV1AdminNamespacesNamespaceEventsGet.create(
        device_type=device_type,
        end_time=end_time,
        event_id=event_id,
        event_name=event_name,
        event_payload=event_payload,
        flight_id=flight_id,
        limit=limit,
        offset=offset,
        start_time=start_time,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetEventsGameTelemetryV1AdminNamespacesNamespaceEventsGet)
async def get_events_game_telemetry_v1_admin_namespaces_namespace_events_get_async(
    device_type: Optional[str] = None,
    end_time: Optional[str] = None,
    event_id: Optional[str] = None,
    event_name: Optional[str] = None,
    event_payload: Optional[str] = None,
    flight_id: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    start_time: Optional[str] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Events (get_events_game_telemetry_v1_admin_namespaces__namespace__events_get)

    This endpoint requires valid JWT token and telemetry permission
    This endpoint retrieves event list

    Properties:
        url: /game-telemetry/v1/admin/namespaces/{namespace}/events

        method: GET

        tags: ["telemetry"]

        consumes: []

        produces: ["application/json"]

        securities: [COOKIE_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        device_type: (deviceType) OPTIONAL str in query

        end_time: (endTime) OPTIONAL str in query

        event_id: (eventId) OPTIONAL str in query

        event_name: (eventName) OPTIONAL str in query

        event_payload: (eventPayload) OPTIONAL str in query

        flight_id: (flightId) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        start_time: (startTime) OPTIONAL str in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - PagedResponseGetNamespaceEventResponse (Successful Response)

        400: Bad Request - BaseErrorResponse (Bad Request)

        422: Unprocessable Entity - HTTPValidationError (Validation Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventsGameTelemetryV1AdminNamespacesNamespaceEventsGet.create(
        device_type=device_type,
        end_time=end_time,
        event_id=event_id,
        event_name=event_name,
        event_payload=event_payload,
        flight_id=flight_id,
        limit=limit,
        offset=offset,
        start_time=start_time,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetNamespacesGameTelemetryV1AdminNamespacesGet)
def get_namespaces_game_telemetry_v1_admin_namespaces_get(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get Namespaces (get_namespaces_game_telemetry_v1_admin_namespaces_get)

    This endpoint requires valid JWT token and telemetry permission
    This endpoint retrieves namespace list

    Properties:
        url: /game-telemetry/v1/admin/namespaces

        method: GET

        tags: ["telemetry"]

        consumes: []

        produces: ["application/json"]

        securities: [COOKIE_AUTH] or [BEARER_AUTH]

    Responses:
        200: OK - ListBaseResponseStr (Successful Response)

        500: Internal Server Error - BaseErrorResponse (Internal Server Error)
    """
    request = GetNamespacesGameTelemetryV1AdminNamespacesGet.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetNamespacesGameTelemetryV1AdminNamespacesGet)
async def get_namespaces_game_telemetry_v1_admin_namespaces_get_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get Namespaces (get_namespaces_game_telemetry_v1_admin_namespaces_get)

    This endpoint requires valid JWT token and telemetry permission
    This endpoint retrieves namespace list

    Properties:
        url: /game-telemetry/v1/admin/namespaces

        method: GET

        tags: ["telemetry"]

        consumes: []

        produces: ["application/json"]

        securities: [COOKIE_AUTH] or [BEARER_AUTH]

    Responses:
        200: OK - ListBaseResponseStr (Successful Response)

        500: Internal Server Error - BaseErrorResponse (Internal Server Error)
    """
    request = GetNamespacesGameTelemetryV1AdminNamespacesGet.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.accountcommon_override_role_permission import (
    AccountcommonOverrideRolePermission,
)
from ..models.accountcommon_replace_role_permission import (
    AccountcommonReplaceRolePermission,
)


class ModelRoleOverrideUpdateRequest(Model):
    """Model role override update request (model.RoleOverrideUpdateRequest)

    Properties:
        additions: (additions) REQUIRED List[AccountcommonOverrideRolePermission]

        exclusions: (exclusions) REQUIRED List[AccountcommonOverrideRolePermission]

        overrides: (overrides) REQUIRED List[AccountcommonOverrideRolePermission]

        replacements: (replacements) REQUIRED List[AccountcommonReplaceRolePermission]
    """

    # region fields

    additions: List[AccountcommonOverrideRolePermission]  # REQUIRED
    exclusions: List[AccountcommonOverrideRolePermission]  # REQUIRED
    overrides: List[AccountcommonOverrideRolePermission]  # REQUIRED
    replacements: List[AccountcommonReplaceRolePermission]  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_additions(
        self, value: List[AccountcommonOverrideRolePermission]
    ) -> ModelRoleOverrideUpdateRequest:
        self.additions = value
        return self

    def with_exclusions(
        self, value: List[AccountcommonOverrideRolePermission]
    ) -> ModelRoleOverrideUpdateRequest:
        self.exclusions = value
        return self

    def with_overrides(
        self, value: List[AccountcommonOverrideRolePermission]
    ) -> ModelRoleOverrideUpdateRequest:
        self.overrides = value
        return self

    def with_replacements(
        self, value: List[AccountcommonReplaceRolePermission]
    ) -> ModelRoleOverrideUpdateRequest:
        self.replacements = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "additions"):
            result["additions"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.additions
            ]
        elif include_empty:
            result["additions"] = []
        if hasattr(self, "exclusions"):
            result["exclusions"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.exclusions
            ]
        elif include_empty:
            result["exclusions"] = []
        if hasattr(self, "overrides"):
            result["overrides"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.overrides
            ]
        elif include_empty:
            result["overrides"] = []
        if hasattr(self, "replacements"):
            result["replacements"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.replacements
            ]
        elif include_empty:
            result["replacements"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        additions: List[AccountcommonOverrideRolePermission],
        exclusions: List[AccountcommonOverrideRolePermission],
        overrides: List[AccountcommonOverrideRolePermission],
        replacements: List[AccountcommonReplaceRolePermission],
        **kwargs,
    ) -> ModelRoleOverrideUpdateRequest:
        instance = cls()
        instance.additions = additions
        instance.exclusions = exclusions
        instance.overrides = overrides
        instance.replacements = replacements
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelRoleOverrideUpdateRequest:
        instance = cls()
        if not dict_:
            return instance
        if "additions" in dict_ and dict_["additions"] is not None:
            instance.additions = [
                AccountcommonOverrideRolePermission.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["additions"]
            ]
        elif include_empty:
            instance.additions = []
        if "exclusions" in dict_ and dict_["exclusions"] is not None:
            instance.exclusions = [
                AccountcommonOverrideRolePermission.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["exclusions"]
            ]
        elif include_empty:
            instance.exclusions = []
        if "overrides" in dict_ and dict_["overrides"] is not None:
            instance.overrides = [
                AccountcommonOverrideRolePermission.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["overrides"]
            ]
        elif include_empty:
            instance.overrides = []
        if "replacements" in dict_ and dict_["replacements"] is not None:
            instance.replacements = [
                AccountcommonReplaceRolePermission.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["replacements"]
            ]
        elif include_empty:
            instance.replacements = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelRoleOverrideUpdateRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelRoleOverrideUpdateRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelRoleOverrideUpdateRequest,
        List[ModelRoleOverrideUpdateRequest],
        Dict[Any, ModelRoleOverrideUpdateRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "additions": "additions",
            "exclusions": "exclusions",
            "overrides": "overrides",
            "replacements": "replacements",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "additions": True,
            "exclusions": True,
            "overrides": True,
            "replacements": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Ugc Service."""

__version__ = "2.21.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .delete_content_screenshot_v2 import DeleteContentScreenshotV2
from .public_bulk_get_content_a86be5 import PublicBulkGetContentByIDV2
from .public_create_content_v2 import PublicCreateContentV2
from .public_delete_content_b_4792d8 import PublicDeleteContentByShareCodeV2
from .public_delete_content_v2 import PublicDeleteContentV2
from .public_generate_content_ffec28 import PublicGenerateContentUploadURLV2
from .public_get_content_bulk_503347 import PublicGetContentBulkByShareCodesV2
from .public_get_content_by_c_87966e import PublicGetContentByChannelIDV2
from .public_get_content_by_idv2 import PublicGetContentByIDV2
from .public_get_content_by_s_4f7083 import PublicGetContentByShareCodeV2
from .public_get_content_by_u_635e0b import PublicGetContentByUserIDV2
from .public_list_content_v2 import PublicListContentV2
from .public_update_content_b_0c4d9c import PublicUpdateContentByShareCodeV2
from .public_update_content_f_55e48b import PublicUpdateContentFileLocation
from .public_update_content_v2 import PublicUpdateContentV2
from .update_content_share_code_v2 import UpdateContentShareCodeV2
from .update_screenshots_v2 import UpdateScreenshotsV2
from .upload_content_screenshot_v2 import UploadContentScreenshotV2



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Event Log Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsEvent(Model):
    """Models event (models.Event)

    Properties:
        agent_type: (AgentType) REQUIRED int

        client_id: (ClientID) REQUIRED str

        component_id: (ComponentID) REQUIRED int

        event_id: (EventID) REQUIRED int

        event_level: (EventLevel) REQUIRED int

        event_type: (EventType) REQUIRED int

        ip_address: (IPAddress) REQUIRED str

        namespace: (Namespace) REQUIRED str

        realm: (Realm) REQUIRED str

        target_namespace: (TargetNamespace) REQUIRED str

        target_user_id: (TargetUserID) REQUIRED str

        time: (Time) REQUIRED str

        user_id: (UserID) REQUIRED str

        ux: (UX) REQUIRED int
    """

    # region fields

    agent_type: int  # REQUIRED
    client_id: str  # REQUIRED
    component_id: int  # REQUIRED
    event_id: int  # REQUIRED
    event_level: int  # REQUIRED
    event_type: int  # REQUIRED
    ip_address: str  # REQUIRED
    namespace: str  # REQUIRED
    realm: str  # REQUIRED
    target_namespace: str  # REQUIRED
    target_user_id: str  # REQUIRED
    time: str  # REQUIRED
    user_id: str  # REQUIRED
    ux: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_agent_type(self, value: int) -> ModelsEvent:
        self.agent_type = value
        return self

    def with_client_id(self, value: str) -> ModelsEvent:
        self.client_id = value
        return self

    def with_component_id(self, value: int) -> ModelsEvent:
        self.component_id = value
        return self

    def with_event_id(self, value: int) -> ModelsEvent:
        self.event_id = value
        return self

    def with_event_level(self, value: int) -> ModelsEvent:
        self.event_level = value
        return self

    def with_event_type(self, value: int) -> ModelsEvent:
        self.event_type = value
        return self

    def with_ip_address(self, value: str) -> ModelsEvent:
        self.ip_address = value
        return self

    def with_namespace(self, value: str) -> ModelsEvent:
        self.namespace = value
        return self

    def with_realm(self, value: str) -> ModelsEvent:
        self.realm = value
        return self

    def with_target_namespace(self, value: str) -> ModelsEvent:
        self.target_namespace = value
        return self

    def with_target_user_id(self, value: str) -> ModelsEvent:
        self.target_user_id = value
        return self

    def with_time(self, value: str) -> ModelsEvent:
        self.time = value
        return self

    def with_user_id(self, value: str) -> ModelsEvent:
        self.user_id = value
        return self

    def with_ux(self, value: int) -> ModelsEvent:
        self.ux = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "agent_type"):
            result["AgentType"] = int(self.agent_type)
        elif include_empty:
            result["AgentType"] = 0
        if hasattr(self, "client_id"):
            result["ClientID"] = str(self.client_id)
        elif include_empty:
            result["ClientID"] = ""
        if hasattr(self, "component_id"):
            result["ComponentID"] = int(self.component_id)
        elif include_empty:
            result["ComponentID"] = 0
        if hasattr(self, "event_id"):
            result["EventID"] = int(self.event_id)
        elif include_empty:
            result["EventID"] = 0
        if hasattr(self, "event_level"):
            result["EventLevel"] = int(self.event_level)
        elif include_empty:
            result["EventLevel"] = 0
        if hasattr(self, "event_type"):
            result["EventType"] = int(self.event_type)
        elif include_empty:
            result["EventType"] = 0
        if hasattr(self, "ip_address"):
            result["IPAddress"] = str(self.ip_address)
        elif include_empty:
            result["IPAddress"] = ""
        if hasattr(self, "namespace"):
            result["Namespace"] = str(self.namespace)
        elif include_empty:
            result["Namespace"] = ""
        if hasattr(self, "realm"):
            result["Realm"] = str(self.realm)
        elif include_empty:
            result["Realm"] = ""
        if hasattr(self, "target_namespace"):
            result["TargetNamespace"] = str(self.target_namespace)
        elif include_empty:
            result["TargetNamespace"] = ""
        if hasattr(self, "target_user_id"):
            result["TargetUserID"] = str(self.target_user_id)
        elif include_empty:
            result["TargetUserID"] = ""
        if hasattr(self, "time"):
            result["Time"] = str(self.time)
        elif include_empty:
            result["Time"] = ""
        if hasattr(self, "user_id"):
            result["UserID"] = str(self.user_id)
        elif include_empty:
            result["UserID"] = ""
        if hasattr(self, "ux"):
            result["UX"] = int(self.ux)
        elif include_empty:
            result["UX"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        agent_type: int,
        client_id: str,
        component_id: int,
        event_id: int,
        event_level: int,
        event_type: int,
        ip_address: str,
        namespace: str,
        realm: str,
        target_namespace: str,
        target_user_id: str,
        time: str,
        user_id: str,
        ux: int,
        **kwargs,
    ) -> ModelsEvent:
        instance = cls()
        instance.agent_type = agent_type
        instance.client_id = client_id
        instance.component_id = component_id
        instance.event_id = event_id
        instance.event_level = event_level
        instance.event_type = event_type
        instance.ip_address = ip_address
        instance.namespace = namespace
        instance.realm = realm
        instance.target_namespace = target_namespace
        instance.target_user_id = target_user_id
        instance.time = time
        instance.user_id = user_id
        instance.ux = ux
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> ModelsEvent:
        instance = cls()
        if not dict_:
            return instance
        if "AgentType" in dict_ and dict_["AgentType"] is not None:
            instance.agent_type = int(dict_["AgentType"])
        elif include_empty:
            instance.agent_type = 0
        if "ClientID" in dict_ and dict_["ClientID"] is not None:
            instance.client_id = str(dict_["ClientID"])
        elif include_empty:
            instance.client_id = ""
        if "ComponentID" in dict_ and dict_["ComponentID"] is not None:
            instance.component_id = int(dict_["ComponentID"])
        elif include_empty:
            instance.component_id = 0
        if "EventID" in dict_ and dict_["EventID"] is not None:
            instance.event_id = int(dict_["EventID"])
        elif include_empty:
            instance.event_id = 0
        if "EventLevel" in dict_ and dict_["EventLevel"] is not None:
            instance.event_level = int(dict_["EventLevel"])
        elif include_empty:
            instance.event_level = 0
        if "EventType" in dict_ and dict_["EventType"] is not None:
            instance.event_type = int(dict_["EventType"])
        elif include_empty:
            instance.event_type = 0
        if "IPAddress" in dict_ and dict_["IPAddress"] is not None:
            instance.ip_address = str(dict_["IPAddress"])
        elif include_empty:
            instance.ip_address = ""
        if "Namespace" in dict_ and dict_["Namespace"] is not None:
            instance.namespace = str(dict_["Namespace"])
        elif include_empty:
            instance.namespace = ""
        if "Realm" in dict_ and dict_["Realm"] is not None:
            instance.realm = str(dict_["Realm"])
        elif include_empty:
            instance.realm = ""
        if "TargetNamespace" in dict_ and dict_["TargetNamespace"] is not None:
            instance.target_namespace = str(dict_["TargetNamespace"])
        elif include_empty:
            instance.target_namespace = ""
        if "TargetUserID" in dict_ and dict_["TargetUserID"] is not None:
            instance.target_user_id = str(dict_["TargetUserID"])
        elif include_empty:
            instance.target_user_id = ""
        if "Time" in dict_ and dict_["Time"] is not None:
            instance.time = str(dict_["Time"])
        elif include_empty:
            instance.time = ""
        if "UserID" in dict_ and dict_["UserID"] is not None:
            instance.user_id = str(dict_["UserID"])
        elif include_empty:
            instance.user_id = ""
        if "UX" in dict_ and dict_["UX"] is not None:
            instance.ux = int(dict_["UX"])
        elif include_empty:
            instance.ux = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsEvent]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsEvent]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ModelsEvent, List[ModelsEvent], Dict[Any, ModelsEvent]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "AgentType": "agent_type",
            "ClientID": "client_id",
            "ComponentID": "component_id",
            "EventID": "event_id",
            "EventLevel": "event_level",
            "EventType": "event_type",
            "IPAddress": "ip_address",
            "Namespace": "namespace",
            "Realm": "realm",
            "TargetNamespace": "target_namespace",
            "TargetUserID": "target_user_id",
            "Time": "time",
            "UserID": "user_id",
            "UX": "ux",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "AgentType": True,
            "ClientID": True,
            "ComponentID": True,
            "EventID": True,
            "EventLevel": True,
            "EventType": True,
            "IPAddress": True,
            "Namespace": True,
            "Realm": True,
            "TargetNamespace": True,
            "TargetUserID": True,
            "Time": True,
            "UserID": True,
            "UX": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import DtoListFinishedDataRequests
from ..models import ModelsS2SDataRetrievalResponse
from ..models import ModelsS2SUserDataURL
from ..models import ResponseError

from ..operations.data_retrieval_s2s import S2SGeneratePersonalDataURL
from ..operations.data_retrieval_s2s import S2SGetListFinishedPersonalDataRequest
from ..operations.data_retrieval_s2s import S2SRequestDataRetrieval


@same_doc_as(S2SGeneratePersonalDataURL)
def s2s_generate_personal_data_url(
    request_date: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Generate personal data download url (S2SGeneratePersonalDataURL)

    Scope: account

    Generate personal data download url.

    **Limitation:** This API only accepts requests with a publisher userId and does not support game userId requests at this time.

    ---
    ## This API for S2S integration purpose only

    Properties:
        url: /gdpr/s2s/namespaces/{namespace}/users/{userId}/requests/{requestDate}/generate

        method: POST

        tags: ["Data Retrieval (S2S)"]

        consumes: ["*/*"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        request_date: (requestDate) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsS2SUserDataURL (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = S2SGeneratePersonalDataURL.create(
        request_date=request_date,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(S2SGeneratePersonalDataURL)
async def s2s_generate_personal_data_url_async(
    request_date: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Generate personal data download url (S2SGeneratePersonalDataURL)

    Scope: account

    Generate personal data download url.

    **Limitation:** This API only accepts requests with a publisher userId and does not support game userId requests at this time.

    ---
    ## This API for S2S integration purpose only

    Properties:
        url: /gdpr/s2s/namespaces/{namespace}/users/{userId}/requests/{requestDate}/generate

        method: POST

        tags: ["Data Retrieval (S2S)"]

        consumes: ["*/*"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        request_date: (requestDate) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsS2SUserDataURL (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = S2SGeneratePersonalDataURL.create(
        request_date=request_date,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(S2SGetListFinishedPersonalDataRequest)
def s2s_get_list_finished_personal_data_request(
    end: str,
    start: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of finished personal data requests (S2SGetListFinishedPersonalDataRequest)

    Scope: account

    Get list of finished personal data requests based on the finished time period.
    Unfinished personal data requests will not appear here, i.e. have Status **Pending**, **In-Progress** or **Canceled**.

    **Anonymize userId for deleted account:**
    For user accounts that have been deleted, the **userId** field in this API will be anonymized automatically after **7 days** from the success deletion.
    This measure is implemented to ensure compliance with GDPR regulations. Please make sure to synchronize the data from this API before it undergoes anonymization.

    ---
    ## This API for S2S integration purpose only

    Properties:
        url: /gdpr/s2s/namespaces/{namespace}/requests/finished

        method: GET

        tags: ["Data Retrieval (S2S)"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end: (end) REQUIRED str in query

        start: (start) REQUIRED str in query

    Responses:
        200: OK - DtoListFinishedDataRequests (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = S2SGetListFinishedPersonalDataRequest.create(
        end=end,
        start=start,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(S2SGetListFinishedPersonalDataRequest)
async def s2s_get_list_finished_personal_data_request_async(
    end: str,
    start: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of finished personal data requests (S2SGetListFinishedPersonalDataRequest)

    Scope: account

    Get list of finished personal data requests based on the finished time period.
    Unfinished personal data requests will not appear here, i.e. have Status **Pending**, **In-Progress** or **Canceled**.

    **Anonymize userId for deleted account:**
    For user accounts that have been deleted, the **userId** field in this API will be anonymized automatically after **7 days** from the success deletion.
    This measure is implemented to ensure compliance with GDPR regulations. Please make sure to synchronize the data from this API before it undergoes anonymization.

    ---
    ## This API for S2S integration purpose only

    Properties:
        url: /gdpr/s2s/namespaces/{namespace}/requests/finished

        method: GET

        tags: ["Data Retrieval (S2S)"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end: (end) REQUIRED str in query

        start: (start) REQUIRED str in query

    Responses:
        200: OK - DtoListFinishedDataRequests (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = S2SGetListFinishedPersonalDataRequest.create(
        end=end,
        start=start,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(S2SRequestDataRetrieval)
def s2s_request_data_retrieval(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Submit user personal data retrieval request (S2SRequestDataRetrieval)

    Scope: account

    Submit user personal data retrieval request.

    **Limitation:** This API only accepts requests with a publisher userId and does not support game userId requests at this time.

    ---
    ## This API for S2S integration purpose only

    **Notes:**

    1. This API will **not send GDPR email notification** both for player and admin notification.

    Properties:
        url: /gdpr/s2s/namespaces/{namespace}/users/{userId}/requests

        method: POST

        tags: ["Data Retrieval (S2S)"]

        consumes: ["*/*"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - ModelsS2SDataRetrievalResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = S2SRequestDataRetrieval.create(
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(S2SRequestDataRetrieval)
async def s2s_request_data_retrieval_async(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Submit user personal data retrieval request (S2SRequestDataRetrieval)

    Scope: account

    Submit user personal data retrieval request.

    **Limitation:** This API only accepts requests with a publisher userId and does not support game userId requests at this time.

    ---
    ## This API for S2S integration purpose only

    **Notes:**

    1. This API will **not send GDPR email notification** both for player and admin notification.

    Properties:
        url: /gdpr/s2s/namespaces/{namespace}/users/{userId}/requests

        method: POST

        tags: ["Data Retrieval (S2S)"]

        consumes: ["*/*"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - ModelsS2SDataRetrievalResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = S2SRequestDataRetrieval.create(
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Session History Service."""

__version__ = "1.12.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

# config
from .wrappers import admin_get_log_config
from .wrappers import admin_get_log_config_async
from .wrappers import admin_patch_update_log_config
from .wrappers import admin_patch_update_log_config_async

# game_session_detail
from .wrappers import admin_get_matchmaking_detail_by_session_id
from .wrappers import admin_get_matchmaking_detail_by_session_id_async
from .wrappers import admin_get_matchmaking_detail_by_ticket_id
from .wrappers import admin_get_matchmaking_detail_by_ticket_id_async
from .wrappers import admin_query_game_session_detail
from .wrappers import admin_query_game_session_detail_async
from .wrappers import admin_query_matchmaking_detail
from .wrappers import admin_query_matchmaking_detail_async
from .wrappers import admin_query_party_detail
from .wrappers import admin_query_party_detail_async
from .wrappers import admin_query_ticket_detail
from .wrappers import admin_query_ticket_detail_async
from .wrappers import admin_ticket_detail_get_by_ticket_id
from .wrappers import admin_ticket_detail_get_by_ticket_id_async
from .wrappers import get_game_session_detail
from .wrappers import get_game_session_detail_async
from .wrappers import get_party_detail
from .wrappers import get_party_detail_async
from .wrappers import public_query_game_session_me
from .wrappers import public_query_game_session_me_async

# operations
from .wrappers import get_healthcheck_info
from .wrappers import get_healthcheck_info_async
from .wrappers import get_healthcheck_info_v1
from .wrappers import get_healthcheck_info_v1_async

# x_ray
from .wrappers import create_xray_bulk_ticket_observability
from .wrappers import create_xray_bulk_ticket_observability_async
from .wrappers import create_xray_ticket_observability
from .wrappers import create_xray_ticket_observability_async
from .wrappers import query_acquiring_ds
from .wrappers import query_acquiring_ds_async
from .wrappers import query_acquiring_ds_wait_time_avg
from .wrappers import query_acquiring_ds_wait_time_avg_async
from .wrappers import query_detail_tick_match_pool
from .wrappers import query_detail_tick_match_pool_async
from .wrappers import query_detail_tick_match_pool_matches
from .wrappers import query_detail_tick_match_pool_matches_async
from .wrappers import query_detail_tick_match_pool_ticket
from .wrappers import query_detail_tick_match_pool_ticket_async
from .wrappers import query_match_histories
from .wrappers import query_match_histories_async
from .wrappers import query_match_length_durationp99
from .wrappers import query_match_length_durationp99_async
from .wrappers import query_match_length_durationp_avg
from .wrappers import query_match_length_durationp_avg_async
from .wrappers import query_match_ticket_histories
from .wrappers import query_match_ticket_histories_async
from .wrappers import query_total_active_session
from .wrappers import query_total_active_session_async
from .wrappers import query_total_matchmaking_canceled
from .wrappers import query_total_matchmaking_canceled_async
from .wrappers import query_total_matchmaking_created
from .wrappers import query_total_matchmaking_created_async
from .wrappers import query_total_matchmaking_expired
from .wrappers import query_total_matchmaking_expired_async
from .wrappers import query_total_matchmaking_match
from .wrappers import query_total_matchmaking_match_async
from .wrappers import query_total_matchmaking_match_ticket
from .wrappers import query_total_matchmaking_match_ticket_async
from .wrappers import query_total_player_persession
from .wrappers import query_total_player_persession_async
from .wrappers import query_xray_match
from .wrappers import query_xray_match_async
from .wrappers import query_xray_match_pool
from .wrappers import query_xray_match_pool_async
from .wrappers import query_xray_timeline_by_ticket_id
from .wrappers import query_xray_timeline_by_ticket_id_async
from .wrappers import query_xray_timeline_by_user_id
from .wrappers import query_xray_timeline_by_user_id_async



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApimodelsErrorResponse
from ..models import ApimodelsListInventoryResp

from ..operations.public_inventories import PublicListInventories
from ..operations.public_inventories import PublicListInventoriesSortByEnum


@same_doc_as(PublicListInventories)
def public_list_inventories(
    inventory_configuration_code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[Union[str, PublicListInventoriesSortByEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To list all my inventories (PublicListInventories)


    Listing all my inventories in a namespace.
    The response body will be in the form of standard pagination.

    Properties:
        url: /inventory/v1/public/namespaces/{namespace}/users/me/inventories

        method: GET

        tags: ["Public Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        inventory_configuration_code: (inventoryConfigurationCode) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

    Responses:
        200: OK - ApimodelsListInventoryResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicListInventories.create(
        inventory_configuration_code=inventory_configuration_code,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicListInventories)
async def public_list_inventories_async(
    inventory_configuration_code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[Union[str, PublicListInventoriesSortByEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To list all my inventories (PublicListInventories)


    Listing all my inventories in a namespace.
    The response body will be in the form of standard pagination.

    Properties:
        url: /inventory/v1/public/namespaces/{namespace}/users/me/inventories

        method: GET

        tags: ["Public Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        inventory_configuration_code: (inventoryConfigurationCode) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

    Responses:
        200: OK - ApimodelsListInventoryResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicListInventories.create(
        inventory_configuration_code=inventory_configuration_code,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import Customization
from ..models import ErrorEntity
from ..models import PaymentMethod
from ..models import PaymentOrderDetails
from ..models import PaymentOrderPaidResult
from ..models import PaymentProcessResult
from ..models import PaymentToken
from ..models import PaymentUrl
from ..models import PaymentUrlCreate
from ..models import TaxResult

from ..operations.payment_station import GetPaymentCustomization
from ..operations.payment_station import GetPaymentCustomizationPaymentProviderEnum
from ..operations.payment_station import GetPaymentPublicConfig
from ..operations.payment_station import GetPaymentPublicConfigPaymentProviderEnum
from ..operations.payment_station import GetPaymentTaxValue
from ..operations.payment_station import GetPaymentTaxValuePaymentProviderEnum
from ..operations.payment_station import Pay
from ..operations.payment_station import PayPaymentProviderEnum
from ..operations.payment_station import PublicCheckPaymentOrderPaidStatus
from ..operations.payment_station import PublicGetPaymentMethods
from ..operations.payment_station import PublicGetPaymentUrl
from ..operations.payment_station import PublicGetQRCode
from ..operations.payment_station import PublicGetUnpaidPaymentOrder
from ..operations.payment_station import PublicNormalizePaymentReturnUrl
from ..operations.payment_station import (
    PublicNormalizePaymentReturnUrlPaymentProviderEnum,
)
from ..models import PaymentMethodPaymentProviderEnum
from ..models import PaymentUrlPaymentProviderEnum, PaymentUrlPaymentTypeEnum
from ..models import PaymentUrlCreatePaymentProviderEnum


@deprecated
@same_doc_as(GetPaymentCustomization)
def get_payment_customization(
    payment_provider: Union[str, GetPaymentCustomizationPaymentProviderEnum],
    region: str,
    sandbox: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment provider customization (getPaymentCustomization)

    [Not supported yet in AGS Shared Cloud] Get payment provider customization, at current only Adyen provide customization. This api has been deprecated, pls use /public/namespaces/{namespace}/payment/publicconfig to get adyen config
    Other detail info:

      * Returns : customization

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/customization

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        sandbox: (sandbox) OPTIONAL bool in query

        payment_provider: (paymentProvider) REQUIRED Union[str, PaymentProviderEnum] in query

        region: (region) REQUIRED str in query

    Responses:
        200: OK - Customization (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentCustomization.create(
        payment_provider=payment_provider,
        region=region,
        sandbox=sandbox,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetPaymentCustomization)
async def get_payment_customization_async(
    payment_provider: Union[str, GetPaymentCustomizationPaymentProviderEnum],
    region: str,
    sandbox: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment provider customization (getPaymentCustomization)

    [Not supported yet in AGS Shared Cloud] Get payment provider customization, at current only Adyen provide customization. This api has been deprecated, pls use /public/namespaces/{namespace}/payment/publicconfig to get adyen config
    Other detail info:

      * Returns : customization

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/customization

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        sandbox: (sandbox) OPTIONAL bool in query

        payment_provider: (paymentProvider) REQUIRED Union[str, PaymentProviderEnum] in query

        region: (region) REQUIRED str in query

    Responses:
        200: OK - Customization (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentCustomization.create(
        payment_provider=payment_provider,
        region=region,
        sandbox=sandbox,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPaymentPublicConfig)
def get_payment_public_config(
    payment_provider: Union[str, GetPaymentPublicConfigPaymentProviderEnum],
    region: str,
    sandbox: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment provider public config (getPaymentPublicConfig)

    [Not supported yet in AGS Shared Cloud] Get payment provider public config, at current only Strip provide public config.
    Other detail info:

      * Returns : Public config

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/publicconfig

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        sandbox: (sandbox) OPTIONAL bool in query

        payment_provider: (paymentProvider) REQUIRED Union[str, PaymentProviderEnum] in query

        region: (region) REQUIRED str in query

    Responses:
        200: OK - Dict[str, Any] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentPublicConfig.create(
        payment_provider=payment_provider,
        region=region,
        sandbox=sandbox,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPaymentPublicConfig)
async def get_payment_public_config_async(
    payment_provider: Union[str, GetPaymentPublicConfigPaymentProviderEnum],
    region: str,
    sandbox: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment provider public config (getPaymentPublicConfig)

    [Not supported yet in AGS Shared Cloud] Get payment provider public config, at current only Strip provide public config.
    Other detail info:

      * Returns : Public config

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/publicconfig

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        sandbox: (sandbox) OPTIONAL bool in query

        payment_provider: (paymentProvider) REQUIRED Union[str, PaymentProviderEnum] in query

        region: (region) REQUIRED str in query

    Responses:
        200: OK - Dict[str, Any] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentPublicConfig.create(
        payment_provider=payment_provider,
        region=region,
        sandbox=sandbox,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPaymentTaxValue)
def get_payment_tax_value(
    payment_order_no: str,
    payment_provider: Union[str, GetPaymentTaxValuePaymentProviderEnum],
    zip_code: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get tax result of a payment order (getPaymentTaxValue)

    [Not supported yet in AGS Shared Cloud] Check and get a payment order's should pay tax.
    Other detail info:

      * Returns : tax result

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/tax

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        zip_code: (zipCode) OPTIONAL str in query

        payment_order_no: (paymentOrderNo) REQUIRED str in query

        payment_provider: (paymentProvider) REQUIRED Union[str, PaymentProviderEnum] in query

    Responses:
        200: OK - TaxResult (successful operation)

        400: Bad Request - ErrorEntity (33123: Invalid zipcode)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentTaxValue.create(
        payment_order_no=payment_order_no,
        payment_provider=payment_provider,
        zip_code=zip_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPaymentTaxValue)
async def get_payment_tax_value_async(
    payment_order_no: str,
    payment_provider: Union[str, GetPaymentTaxValuePaymentProviderEnum],
    zip_code: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get tax result of a payment order (getPaymentTaxValue)

    [Not supported yet in AGS Shared Cloud] Check and get a payment order's should pay tax.
    Other detail info:

      * Returns : tax result

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/tax

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        zip_code: (zipCode) OPTIONAL str in query

        payment_order_no: (paymentOrderNo) REQUIRED str in query

        payment_provider: (paymentProvider) REQUIRED Union[str, PaymentProviderEnum] in query

    Responses:
        200: OK - TaxResult (successful operation)

        400: Bad Request - ErrorEntity (33123: Invalid zipcode)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentTaxValue.create(
        payment_order_no=payment_order_no,
        payment_provider=payment_provider,
        zip_code=zip_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(Pay)
def pay(
    body: PaymentToken,
    payment_order_no: str,
    payment_provider: Optional[Union[str, PayPaymentProviderEnum]] = None,
    zip_code: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Do payment (pay)

    [Not supported yet in AGS Shared Cloud] Do payment(For now, this only support checkout.com).
    Other detail info:

      * Returns : Payment process result

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/orders/{paymentOrderNo}/pay

        method: POST

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED PaymentToken in body

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

        payment_provider: (paymentProvider) OPTIONAL Union[str, PaymentProviderEnum] in query

        zip_code: (zipCode) OPTIONAL str in query

    Responses:
        200: OK - PaymentProcessResult (successful operation)

        400: Bad Request - ErrorEntity (33322: Payment provider [{paymentProvider}] not supported)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)

        409: Conflict - ErrorEntity (33171: Invalid payment order status [{status}] for payment order [{paymentOrderNo}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Pay.create(
        body=body,
        payment_order_no=payment_order_no,
        payment_provider=payment_provider,
        zip_code=zip_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(Pay)
async def pay_async(
    body: PaymentToken,
    payment_order_no: str,
    payment_provider: Optional[Union[str, PayPaymentProviderEnum]] = None,
    zip_code: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Do payment (pay)

    [Not supported yet in AGS Shared Cloud] Do payment(For now, this only support checkout.com).
    Other detail info:

      * Returns : Payment process result

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/orders/{paymentOrderNo}/pay

        method: POST

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED PaymentToken in body

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

        payment_provider: (paymentProvider) OPTIONAL Union[str, PaymentProviderEnum] in query

        zip_code: (zipCode) OPTIONAL str in query

    Responses:
        200: OK - PaymentProcessResult (successful operation)

        400: Bad Request - ErrorEntity (33322: Payment provider [{paymentProvider}] not supported)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)

        409: Conflict - ErrorEntity (33171: Invalid payment order status [{status}] for payment order [{paymentOrderNo}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Pay.create(
        body=body,
        payment_order_no=payment_order_no,
        payment_provider=payment_provider,
        zip_code=zip_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicCheckPaymentOrderPaidStatus)
def public_check_payment_order_paid_status(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Check payment order paid status (publicCheckPaymentOrderPaidStatus)

    [Not supported yet in AGS Shared Cloud] Check payment order paid status.
    Other detail info:

      * Returns : Payment order paid result

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/orders/{paymentOrderNo}/status

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderPaidResult (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicCheckPaymentOrderPaidStatus.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicCheckPaymentOrderPaidStatus)
async def public_check_payment_order_paid_status_async(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Check payment order paid status (publicCheckPaymentOrderPaidStatus)

    [Not supported yet in AGS Shared Cloud] Check payment order paid status.
    Other detail info:

      * Returns : Payment order paid result

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/orders/{paymentOrderNo}/status

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderPaidResult (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicCheckPaymentOrderPaidStatus.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetPaymentMethods)
def public_get_payment_methods(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment methods (publicGetPaymentMethods)

    [Not supported yet in AGS Shared Cloud] Get payment methods.
    Other detail info:

      * Returns : Payment method list

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/methods

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in query

    Responses:
        200: OK - List[PaymentMethod] (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetPaymentMethods.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetPaymentMethods)
async def public_get_payment_methods_async(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment methods (publicGetPaymentMethods)

    [Not supported yet in AGS Shared Cloud] Get payment methods.
    Other detail info:

      * Returns : Payment method list

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/methods

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in query

    Responses:
        200: OK - List[PaymentMethod] (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetPaymentMethods.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetPaymentUrl)
def public_get_payment_url(
    body: PaymentUrlCreate,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment url (publicGetPaymentUrl)

    [Not supported yet in AGS Shared Cloud] Get payment url.
    Other detail info:

      * For Neon Pay payment provider, the 'neonPayConfig' field can be used to provide success and cancel URL. If 'neonPayConfig' field is not present, the 'returnUrl' will be used for both success and cancel URL.
      *  Returns : Get payment link

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/link

        method: POST

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED PaymentUrlCreate in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - PaymentUrl (successful operation)

        400: Bad Request - ErrorEntity (33321: Payment provider [{paymentProvider}] not support currency [{currency}] | 33322: Payment provider [{paymentProvider}] not supported | 33332: Amount too small, please contact administrator | 33333: Neon Pay checkout payment order [{paymentOrderNo}] failed with message [{errMsg}])

        403: Forbidden - ErrorEntity (20016: action is banned)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist | 20008: user [{userId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetPaymentUrl.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetPaymentUrl)
async def public_get_payment_url_async(
    body: PaymentUrlCreate,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment url (publicGetPaymentUrl)

    [Not supported yet in AGS Shared Cloud] Get payment url.
    Other detail info:

      * For Neon Pay payment provider, the 'neonPayConfig' field can be used to provide success and cancel URL. If 'neonPayConfig' field is not present, the 'returnUrl' will be used for both success and cancel URL.
      *  Returns : Get payment link

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/link

        method: POST

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED PaymentUrlCreate in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - PaymentUrl (successful operation)

        400: Bad Request - ErrorEntity (33321: Payment provider [{paymentProvider}] not support currency [{currency}] | 33322: Payment provider [{paymentProvider}] not supported | 33332: Amount too small, please contact administrator | 33333: Neon Pay checkout payment order [{paymentOrderNo}] failed with message [{errMsg}])

        403: Forbidden - ErrorEntity (20016: action is banned)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist | 20008: user [{userId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetPaymentUrl.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetQRCode)
def public_get_qr_code(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get qrcode (publicGetQRCode)

    [Not supported yet in AGS Shared Cloud] Get qrcode.
    Other detail info:

      * Returns : QRCode image stream

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/qrcode

        method: GET

        tags: ["PaymentStation"]

        consumes: []

        produces: ["image/png"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        code: (code) REQUIRED str in query

    Responses:
        200: OK - Any (Successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetQRCode.create(
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetQRCode)
async def public_get_qr_code_async(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get qrcode (publicGetQRCode)

    [Not supported yet in AGS Shared Cloud] Get qrcode.
    Other detail info:

      * Returns : QRCode image stream

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/qrcode

        method: GET

        tags: ["PaymentStation"]

        consumes: []

        produces: ["image/png"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        code: (code) REQUIRED str in query

    Responses:
        200: OK - Any (Successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetQRCode.create(
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetUnpaidPaymentOrder)
def public_get_unpaid_payment_order(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment order info (publicGetUnpaidPaymentOrder)

    [Not supported yet in AGS Shared Cloud] Get payment order info.
    Other detail info:

      * Returns : Payment order details

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/orders/{paymentOrderNo}/info

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderDetails (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist | 20008: user [{userId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (33171: Invalid payment order status [{status}] for payment order [{paymentOrderNo}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetUnpaidPaymentOrder.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetUnpaidPaymentOrder)
async def public_get_unpaid_payment_order_async(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment order info (publicGetUnpaidPaymentOrder)

    [Not supported yet in AGS Shared Cloud] Get payment order info.
    Other detail info:

      * Returns : Payment order details

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/orders/{paymentOrderNo}/info

        method: GET

        tags: ["PaymentStation"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderDetails (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist | 20008: user [{userId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (33171: Invalid payment order status [{status}] for payment order [{paymentOrderNo}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetUnpaidPaymentOrder.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicNormalizePaymentReturnUrl)
def public_normalize_payment_return_url(
    order_no: str,
    payment_order_no: str,
    payment_provider: Union[str, PublicNormalizePaymentReturnUrlPaymentProviderEnum],
    return_url: str,
    foreinginvoice: Optional[str] = None,
    invoice_id: Optional[str] = None,
    payer_id: Optional[str] = None,
    payload: Optional[str] = None,
    redirect_result: Optional[str] = None,
    result_code: Optional[str] = None,
    session_id: Optional[str] = None,
    status: Optional[str] = None,
    token: Optional[str] = None,
    type_: Optional[str] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Normalize payment return url (publicNormalizePaymentReturnUrl)

    [Not supported yet in AGS Shared Cloud] Normalize payment return url for payment provider
    Field                                                                                                                                                        | Type   | Required | Description
    -------------------------------------------------------------------------------------------------------------------------------------------------------------|--------|----------|-----------------------
    orderNo                                                                                                                                                      | String | Yes      | order no
    paymentStatus                                                                                                                                                | String | Yes      |

      * DONE: The payment was successfully completed.
      * CANCELLED: The payment was cancelled by the shopper before completion, or the shopper returned to the merchant's site before completing the transaction.
      * PENDING: Inform the shopper that you've received their order, and are waiting for the payment to be completed.

    When the shopper has completed the payment you will receive a successful AUTHORISATION.
      * RECEIVED: Inform the shopper that you've received their order, and are waiting for the payment to clear.
      * UNKNOWN: An error occurred during the payment processing.
      * FAILED: Shopper paid failed because of various reasons.


    reason                                                                                                                                                       | String | No       | payment status reason

    Other detail info:

      * xsolla : parameters 'user_id', 'foreinginvoice', 'invoice_id' and 'status' will be automatically added to the link
      *  adyen : https://docs.adyen.com/developers/checkout/web-sdk

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/returnurl

        method: GET

        tags: ["PaymentStation"]

        consumes: []

        produces: []

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payer_id: (PayerID) OPTIONAL str in query

        foreinginvoice: (foreinginvoice) OPTIONAL str in query

        invoice_id: (invoice_id) OPTIONAL str in query

        payload: (payload) OPTIONAL str in query

        redirect_result: (redirectResult) OPTIONAL str in query

        result_code: (resultCode) OPTIONAL str in query

        session_id: (sessionId) OPTIONAL str in query

        status: (status) OPTIONAL str in query

        token: (token) OPTIONAL str in query

        type_: (type) OPTIONAL str in query

        user_id: (user_id) OPTIONAL str in query

        order_no: (orderNo) REQUIRED str in query

        payment_order_no: (paymentOrderNo) REQUIRED str in query

        payment_provider: (paymentProvider) REQUIRED Union[str, PaymentProviderEnum] in query

        return_url: (returnUrl) REQUIRED str in query

    Responses:
        204: No Content - (no content.(if query param 'returnUrl' is empty))

        307: Temporary Redirect - (successful operation.(if query param 'returnUrl' is not empty))
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicNormalizePaymentReturnUrl.create(
        order_no=order_no,
        payment_order_no=payment_order_no,
        payment_provider=payment_provider,
        return_url=return_url,
        foreinginvoice=foreinginvoice,
        invoice_id=invoice_id,
        payer_id=payer_id,
        payload=payload,
        redirect_result=redirect_result,
        result_code=result_code,
        session_id=session_id,
        status=status,
        token=token,
        type_=type_,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicNormalizePaymentReturnUrl)
async def public_normalize_payment_return_url_async(
    order_no: str,
    payment_order_no: str,
    payment_provider: Union[str, PublicNormalizePaymentReturnUrlPaymentProviderEnum],
    return_url: str,
    foreinginvoice: Optional[str] = None,
    invoice_id: Optional[str] = None,
    payer_id: Optional[str] = None,
    payload: Optional[str] = None,
    redirect_result: Optional[str] = None,
    result_code: Optional[str] = None,
    session_id: Optional[str] = None,
    status: Optional[str] = None,
    token: Optional[str] = None,
    type_: Optional[str] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Normalize payment return url (publicNormalizePaymentReturnUrl)

    [Not supported yet in AGS Shared Cloud] Normalize payment return url for payment provider
    Field                                                                                                                                                        | Type   | Required | Description
    -------------------------------------------------------------------------------------------------------------------------------------------------------------|--------|----------|-----------------------
    orderNo                                                                                                                                                      | String | Yes      | order no
    paymentStatus                                                                                                                                                | String | Yes      |

      * DONE: The payment was successfully completed.
      * CANCELLED: The payment was cancelled by the shopper before completion, or the shopper returned to the merchant's site before completing the transaction.
      * PENDING: Inform the shopper that you've received their order, and are waiting for the payment to be completed.

    When the shopper has completed the payment you will receive a successful AUTHORISATION.
      * RECEIVED: Inform the shopper that you've received their order, and are waiting for the payment to clear.
      * UNKNOWN: An error occurred during the payment processing.
      * FAILED: Shopper paid failed because of various reasons.


    reason                                                                                                                                                       | String | No       | payment status reason

    Other detail info:

      * xsolla : parameters 'user_id', 'foreinginvoice', 'invoice_id' and 'status' will be automatically added to the link
      *  adyen : https://docs.adyen.com/developers/checkout/web-sdk

    Properties:
        url: /platform/public/namespaces/{namespace}/payment/returnurl

        method: GET

        tags: ["PaymentStation"]

        consumes: []

        produces: []

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payer_id: (PayerID) OPTIONAL str in query

        foreinginvoice: (foreinginvoice) OPTIONAL str in query

        invoice_id: (invoice_id) OPTIONAL str in query

        payload: (payload) OPTIONAL str in query

        redirect_result: (redirectResult) OPTIONAL str in query

        result_code: (resultCode) OPTIONAL str in query

        session_id: (sessionId) OPTIONAL str in query

        status: (status) OPTIONAL str in query

        token: (token) OPTIONAL str in query

        type_: (type) OPTIONAL str in query

        user_id: (user_id) OPTIONAL str in query

        order_no: (orderNo) REQUIRED str in query

        payment_order_no: (paymentOrderNo) REQUIRED str in query

        payment_provider: (paymentProvider) REQUIRED Union[str, PaymentProviderEnum] in query

        return_url: (returnUrl) REQUIRED str in query

    Responses:
        204: No Content - (no content.(if query param 'returnUrl' is empty))

        307: Temporary Redirect - (successful operation.(if query param 'returnUrl' is not empty))
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicNormalizePaymentReturnUrl.create(
        order_no=order_no,
        payment_order_no=payment_order_no,
        payment_provider=payment_provider,
        return_url=return_url,
        foreinginvoice=foreinginvoice,
        invoice_id=invoice_id,
        payer_id=payer_id,
        payload=payload,
        redirect_result=redirect_result,
        result_code=result_code,
        session_id=session_id,
        status=status,
        token=token,
        type_=type_,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class OauthmodelCountryLocationResponse(Model):
    """Oauthmodel country location response (oauthmodel.CountryLocationResponse)

    Properties:
        city: (city) REQUIRED str

        country_code: (countryCode) REQUIRED str

        country_name: (countryName) REQUIRED str

        state: (state) REQUIRED str
    """

    # region fields

    city: str  # REQUIRED
    country_code: str  # REQUIRED
    country_name: str  # REQUIRED
    state: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_city(self, value: str) -> OauthmodelCountryLocationResponse:
        self.city = value
        return self

    def with_country_code(self, value: str) -> OauthmodelCountryLocationResponse:
        self.country_code = value
        return self

    def with_country_name(self, value: str) -> OauthmodelCountryLocationResponse:
        self.country_name = value
        return self

    def with_state(self, value: str) -> OauthmodelCountryLocationResponse:
        self.state = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "city"):
            result["city"] = str(self.city)
        elif include_empty:
            result["city"] = ""
        if hasattr(self, "country_code"):
            result["countryCode"] = str(self.country_code)
        elif include_empty:
            result["countryCode"] = ""
        if hasattr(self, "country_name"):
            result["countryName"] = str(self.country_name)
        elif include_empty:
            result["countryName"] = ""
        if hasattr(self, "state"):
            result["state"] = str(self.state)
        elif include_empty:
            result["state"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls, city: str, country_code: str, country_name: str, state: str, **kwargs
    ) -> OauthmodelCountryLocationResponse:
        instance = cls()
        instance.city = city
        instance.country_code = country_code
        instance.country_name = country_name
        instance.state = state
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> OauthmodelCountryLocationResponse:
        instance = cls()
        if not dict_:
            return instance
        if "city" in dict_ and dict_["city"] is not None:
            instance.city = str(dict_["city"])
        elif include_empty:
            instance.city = ""
        if "countryCode" in dict_ and dict_["countryCode"] is not None:
            instance.country_code = str(dict_["countryCode"])
        elif include_empty:
            instance.country_code = ""
        if "countryName" in dict_ and dict_["countryName"] is not None:
            instance.country_name = str(dict_["countryName"])
        elif include_empty:
            instance.country_name = ""
        if "state" in dict_ and dict_["state"] is not None:
            instance.state = str(dict_["state"])
        elif include_empty:
            instance.state = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, OauthmodelCountryLocationResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[OauthmodelCountryLocationResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        OauthmodelCountryLocationResponse,
        List[OauthmodelCountryLocationResponse],
        Dict[Any, OauthmodelCountryLocationResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "city": "city",
            "countryCode": "country_code",
            "countryName": "country_name",
            "state": "state",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "city": True,
            "countryCode": True,
            "countryName": True,
            "state": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_native_session_setting import ModelsNativeSessionSetting


class ApimodelsConfigurationTemplateResponse(Model):
    """Apimodels configuration template response (apimodels.ConfigurationTemplateResponse)

    Properties:
        client_version: (clientVersion) REQUIRED str

        created_at: (createdAt) REQUIRED str

        deployment: (deployment) REQUIRED str

        inactive_timeout: (inactiveTimeout) REQUIRED int

        invite_timeout: (inviteTimeout) REQUIRED int

        joinability: (joinability) REQUIRED str

        last: (last) REQUIRED str

        max_players: (maxPlayers) REQUIRED int

        min_players: (minPlayers) REQUIRED int

        name: (name) REQUIRED str

        namespace: (namespace) REQUIRED str

        persistent: (persistent) REQUIRED bool

        text_chat: (textChat) REQUIRED bool

        type_: (type) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        app_name: (appName) OPTIONAL str

        attributes: (attributes) OPTIONAL Dict[str, Any]

        auto_join: (autoJoin) OPTIONAL bool

        auto_leave_session: (autoLeaveSession) OPTIONAL bool

        custom_urlgrpc: (customURLGRPC) OPTIONAL str

        disable_code_generation: (disableCodeGeneration) OPTIONAL bool

        disable_resend_invite: (disableResendInvite) OPTIONAL bool

        ds_manual_set_ready: (dsManualSetReady) OPTIONAL bool

        ds_source: (dsSource) OPTIONAL str

        enable_secret: (enableSecret) OPTIONAL bool

        fallback_claim_keys: (fallbackClaimKeys) OPTIONAL List[str]

        immutable_storage: (immutableStorage) OPTIONAL bool

        leader_election_grace_period: (leaderElectionGracePeriod) OPTIONAL int

        manual_rejoin: (manualRejoin) OPTIONAL bool

        max_active_sessions: (maxActiveSessions) OPTIONAL int

        native_session_setting: (nativeSessionSetting) OPTIONAL ModelsNativeSessionSetting

        preferred_claim_keys: (preferredClaimKeys) OPTIONAL List[str]

        psn_base_url: (PSNBaseUrl) OPTIONAL str

        requested_regions: (requestedRegions) OPTIONAL List[str]

        tie_teams_session_lifetime: (tieTeamsSessionLifetime) OPTIONAL bool
    """

    # region fields

    client_version: str  # REQUIRED
    created_at: str  # REQUIRED
    deployment: str  # REQUIRED
    inactive_timeout: int  # REQUIRED
    invite_timeout: int  # REQUIRED
    joinability: str  # REQUIRED
    last: str  # REQUIRED
    max_players: int  # REQUIRED
    min_players: int  # REQUIRED
    name: str  # REQUIRED
    namespace: str  # REQUIRED
    persistent: bool  # REQUIRED
    text_chat: bool  # REQUIRED
    type_: str  # REQUIRED
    updated_at: str  # REQUIRED
    app_name: str  # OPTIONAL
    attributes: Dict[str, Any]  # OPTIONAL
    auto_join: bool  # OPTIONAL
    auto_leave_session: bool  # OPTIONAL
    custom_urlgrpc: str  # OPTIONAL
    disable_code_generation: bool  # OPTIONAL
    disable_resend_invite: bool  # OPTIONAL
    ds_manual_set_ready: bool  # OPTIONAL
    ds_source: str  # OPTIONAL
    enable_secret: bool  # OPTIONAL
    fallback_claim_keys: List[str]  # OPTIONAL
    immutable_storage: bool  # OPTIONAL
    leader_election_grace_period: int  # OPTIONAL
    manual_rejoin: bool  # OPTIONAL
    max_active_sessions: int  # OPTIONAL
    native_session_setting: ModelsNativeSessionSetting  # OPTIONAL
    preferred_claim_keys: List[str]  # OPTIONAL
    psn_base_url: str  # OPTIONAL
    requested_regions: List[str]  # OPTIONAL
    tie_teams_session_lifetime: bool  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_client_version(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.client_version = value
        return self

    def with_created_at(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.created_at = value
        return self

    def with_deployment(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.deployment = value
        return self

    def with_inactive_timeout(
        self, value: int
    ) -> ApimodelsConfigurationTemplateResponse:
        self.inactive_timeout = value
        return self

    def with_invite_timeout(self, value: int) -> ApimodelsConfigurationTemplateResponse:
        self.invite_timeout = value
        return self

    def with_joinability(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.joinability = value
        return self

    def with_last(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.last = value
        return self

    def with_max_players(self, value: int) -> ApimodelsConfigurationTemplateResponse:
        self.max_players = value
        return self

    def with_min_players(self, value: int) -> ApimodelsConfigurationTemplateResponse:
        self.min_players = value
        return self

    def with_name(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.name = value
        return self

    def with_namespace(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.namespace = value
        return self

    def with_persistent(self, value: bool) -> ApimodelsConfigurationTemplateResponse:
        self.persistent = value
        return self

    def with_text_chat(self, value: bool) -> ApimodelsConfigurationTemplateResponse:
        self.text_chat = value
        return self

    def with_type(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.type_ = value
        return self

    def with_updated_at(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.updated_at = value
        return self

    def with_app_name(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.app_name = value
        return self

    def with_attributes(
        self, value: Dict[str, Any]
    ) -> ApimodelsConfigurationTemplateResponse:
        self.attributes = value
        return self

    def with_auto_join(self, value: bool) -> ApimodelsConfigurationTemplateResponse:
        self.auto_join = value
        return self

    def with_auto_leave_session(
        self, value: bool
    ) -> ApimodelsConfigurationTemplateResponse:
        self.auto_leave_session = value
        return self

    def with_custom_urlgrpc(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.custom_urlgrpc = value
        return self

    def with_disable_code_generation(
        self, value: bool
    ) -> ApimodelsConfigurationTemplateResponse:
        self.disable_code_generation = value
        return self

    def with_disable_resend_invite(
        self, value: bool
    ) -> ApimodelsConfigurationTemplateResponse:
        self.disable_resend_invite = value
        return self

    def with_ds_manual_set_ready(
        self, value: bool
    ) -> ApimodelsConfigurationTemplateResponse:
        self.ds_manual_set_ready = value
        return self

    def with_ds_source(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.ds_source = value
        return self

    def with_enable_secret(self, value: bool) -> ApimodelsConfigurationTemplateResponse:
        self.enable_secret = value
        return self

    def with_fallback_claim_keys(
        self, value: List[str]
    ) -> ApimodelsConfigurationTemplateResponse:
        self.fallback_claim_keys = value
        return self

    def with_immutable_storage(
        self, value: bool
    ) -> ApimodelsConfigurationTemplateResponse:
        self.immutable_storage = value
        return self

    def with_leader_election_grace_period(
        self, value: int
    ) -> ApimodelsConfigurationTemplateResponse:
        self.leader_election_grace_period = value
        return self

    def with_manual_rejoin(self, value: bool) -> ApimodelsConfigurationTemplateResponse:
        self.manual_rejoin = value
        return self

    def with_max_active_sessions(
        self, value: int
    ) -> ApimodelsConfigurationTemplateResponse:
        self.max_active_sessions = value
        return self

    def with_native_session_setting(
        self, value: ModelsNativeSessionSetting
    ) -> ApimodelsConfigurationTemplateResponse:
        self.native_session_setting = value
        return self

    def with_preferred_claim_keys(
        self, value: List[str]
    ) -> ApimodelsConfigurationTemplateResponse:
        self.preferred_claim_keys = value
        return self

    def with_psn_base_url(self, value: str) -> ApimodelsConfigurationTemplateResponse:
        self.psn_base_url = value
        return self

    def with_requested_regions(
        self, value: List[str]
    ) -> ApimodelsConfigurationTemplateResponse:
        self.requested_regions = value
        return self

    def with_tie_teams_session_lifetime(
        self, value: bool
    ) -> ApimodelsConfigurationTemplateResponse:
        self.tie_teams_session_lifetime = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "client_version"):
            result["clientVersion"] = str(self.client_version)
        elif include_empty:
            result["clientVersion"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "deployment"):
            result["deployment"] = str(self.deployment)
        elif include_empty:
            result["deployment"] = ""
        if hasattr(self, "inactive_timeout"):
            result["inactiveTimeout"] = int(self.inactive_timeout)
        elif include_empty:
            result["inactiveTimeout"] = 0
        if hasattr(self, "invite_timeout"):
            result["inviteTimeout"] = int(self.invite_timeout)
        elif include_empty:
            result["inviteTimeout"] = 0
        if hasattr(self, "joinability"):
            result["joinability"] = str(self.joinability)
        elif include_empty:
            result["joinability"] = ""
        if hasattr(self, "last"):
            result["last"] = str(self.last)
        elif include_empty:
            result["last"] = ""
        if hasattr(self, "max_players"):
            result["maxPlayers"] = int(self.max_players)
        elif include_empty:
            result["maxPlayers"] = 0
        if hasattr(self, "min_players"):
            result["minPlayers"] = int(self.min_players)
        elif include_empty:
            result["minPlayers"] = 0
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "persistent"):
            result["persistent"] = bool(self.persistent)
        elif include_empty:
            result["persistent"] = False
        if hasattr(self, "text_chat"):
            result["textChat"] = bool(self.text_chat)
        elif include_empty:
            result["textChat"] = False
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "app_name"):
            result["appName"] = str(self.app_name)
        elif include_empty:
            result["appName"] = ""
        if hasattr(self, "attributes"):
            result["attributes"] = {str(k0): v0 for k0, v0 in self.attributes.items()}
        elif include_empty:
            result["attributes"] = {}
        if hasattr(self, "auto_join"):
            result["autoJoin"] = bool(self.auto_join)
        elif include_empty:
            result["autoJoin"] = False
        if hasattr(self, "auto_leave_session"):
            result["autoLeaveSession"] = bool(self.auto_leave_session)
        elif include_empty:
            result["autoLeaveSession"] = False
        if hasattr(self, "custom_urlgrpc"):
            result["customURLGRPC"] = str(self.custom_urlgrpc)
        elif include_empty:
            result["customURLGRPC"] = ""
        if hasattr(self, "disable_code_generation"):
            result["disableCodeGeneration"] = bool(self.disable_code_generation)
        elif include_empty:
            result["disableCodeGeneration"] = False
        if hasattr(self, "disable_resend_invite"):
            result["disableResendInvite"] = bool(self.disable_resend_invite)
        elif include_empty:
            result["disableResendInvite"] = False
        if hasattr(self, "ds_manual_set_ready"):
            result["dsManualSetReady"] = bool(self.ds_manual_set_ready)
        elif include_empty:
            result["dsManualSetReady"] = False
        if hasattr(self, "ds_source"):
            result["dsSource"] = str(self.ds_source)
        elif include_empty:
            result["dsSource"] = ""
        if hasattr(self, "enable_secret"):
            result["enableSecret"] = bool(self.enable_secret)
        elif include_empty:
            result["enableSecret"] = False
        if hasattr(self, "fallback_claim_keys"):
            result["fallbackClaimKeys"] = [str(i0) for i0 in self.fallback_claim_keys]
        elif include_empty:
            result["fallbackClaimKeys"] = []
        if hasattr(self, "immutable_storage"):
            result["immutableStorage"] = bool(self.immutable_storage)
        elif include_empty:
            result["immutableStorage"] = False
        if hasattr(self, "leader_election_grace_period"):
            result["leaderElectionGracePeriod"] = int(self.leader_election_grace_period)
        elif include_empty:
            result["leaderElectionGracePeriod"] = 0
        if hasattr(self, "manual_rejoin"):
            result["manualRejoin"] = bool(self.manual_rejoin)
        elif include_empty:
            result["manualRejoin"] = False
        if hasattr(self, "max_active_sessions"):
            result["maxActiveSessions"] = int(self.max_active_sessions)
        elif include_empty:
            result["maxActiveSessions"] = 0
        if hasattr(self, "native_session_setting"):
            result["nativeSessionSetting"] = self.native_session_setting.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["nativeSessionSetting"] = ModelsNativeSessionSetting()
        if hasattr(self, "preferred_claim_keys"):
            result["preferredClaimKeys"] = [str(i0) for i0 in self.preferred_claim_keys]
        elif include_empty:
            result["preferredClaimKeys"] = []
        if hasattr(self, "psn_base_url"):
            result["PSNBaseUrl"] = str(self.psn_base_url)
        elif include_empty:
            result["PSNBaseUrl"] = ""
        if hasattr(self, "requested_regions"):
            result["requestedRegions"] = [str(i0) for i0 in self.requested_regions]
        elif include_empty:
            result["requestedRegions"] = []
        if hasattr(self, "tie_teams_session_lifetime"):
            result["tieTeamsSessionLifetime"] = bool(self.tie_teams_session_lifetime)
        elif include_empty:
            result["tieTeamsSessionLifetime"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        client_version: str,
        created_at: str,
        deployment: str,
        inactive_timeout: int,
        invite_timeout: int,
        joinability: str,
        last: str,
        max_players: int,
        min_players: int,
        name: str,
        namespace: str,
        persistent: bool,
        text_chat: bool,
        type_: str,
        updated_at: str,
        app_name: Optional[str] = None,
        attributes: Optional[Dict[str, Any]] = None,
        auto_join: Optional[bool] = None,
        auto_leave_session: Optional[bool] = None,
        custom_urlgrpc: Optional[str] = None,
        disable_code_generation: Optional[bool] = None,
        disable_resend_invite: Optional[bool] = None,
        ds_manual_set_ready: Optional[bool] = None,
        ds_source: Optional[str] = None,
        enable_secret: Optional[bool] = None,
        fallback_claim_keys: Optional[List[str]] = None,
        immutable_storage: Optional[bool] = None,
        leader_election_grace_period: Optional[int] = None,
        manual_rejoin: Optional[bool] = None,
        max_active_sessions: Optional[int] = None,
        native_session_setting: Optional[ModelsNativeSessionSetting] = None,
        preferred_claim_keys: Optional[List[str]] = None,
        psn_base_url: Optional[str] = None,
        requested_regions: Optional[List[str]] = None,
        tie_teams_session_lifetime: Optional[bool] = None,
        **kwargs,
    ) -> ApimodelsConfigurationTemplateResponse:
        instance = cls()
        instance.client_version = client_version
        instance.created_at = created_at
        instance.deployment = deployment
        instance.inactive_timeout = inactive_timeout
        instance.invite_timeout = invite_timeout
        instance.joinability = joinability
        instance.last = last
        instance.max_players = max_players
        instance.min_players = min_players
        instance.name = name
        instance.namespace = namespace
        instance.persistent = persistent
        instance.text_chat = text_chat
        instance.type_ = type_
        instance.updated_at = updated_at
        if app_name is not None:
            instance.app_name = app_name
        if attributes is not None:
            instance.attributes = attributes
        if auto_join is not None:
            instance.auto_join = auto_join
        if auto_leave_session is not None:
            instance.auto_leave_session = auto_leave_session
        if custom_urlgrpc is not None:
            instance.custom_urlgrpc = custom_urlgrpc
        if disable_code_generation is not None:
            instance.disable_code_generation = disable_code_generation
        if disable_resend_invite is not None:
            instance.disable_resend_invite = disable_resend_invite
        if ds_manual_set_ready is not None:
            instance.ds_manual_set_ready = ds_manual_set_ready
        if ds_source is not None:
            instance.ds_source = ds_source
        if enable_secret is not None:
            instance.enable_secret = enable_secret
        if fallback_claim_keys is not None:
            instance.fallback_claim_keys = fallback_claim_keys
        if immutable_storage is not None:
            instance.immutable_storage = immutable_storage
        if leader_election_grace_period is not None:
            instance.leader_election_grace_period = leader_election_grace_period
        if manual_rejoin is not None:
            instance.manual_rejoin = manual_rejoin
        if max_active_sessions is not None:
            instance.max_active_sessions = max_active_sessions
        if native_session_setting is not None:
            instance.native_session_setting = native_session_setting
        if preferred_claim_keys is not None:
            instance.preferred_claim_keys = preferred_claim_keys
        if psn_base_url is not None:
            instance.psn_base_url = psn_base_url
        if requested_regions is not None:
            instance.requested_regions = requested_regions
        if tie_teams_session_lifetime is not None:
            instance.tie_teams_session_lifetime = tie_teams_session_lifetime
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsConfigurationTemplateResponse:
        instance = cls()
        if not dict_:
            return instance
        if "clientVersion" in dict_ and dict_["clientVersion"] is not None:
            instance.client_version = str(dict_["clientVersion"])
        elif include_empty:
            instance.client_version = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "deployment" in dict_ and dict_["deployment"] is not None:
            instance.deployment = str(dict_["deployment"])
        elif include_empty:
            instance.deployment = ""
        if "inactiveTimeout" in dict_ and dict_["inactiveTimeout"] is not None:
            instance.inactive_timeout = int(dict_["inactiveTimeout"])
        elif include_empty:
            instance.inactive_timeout = 0
        if "inviteTimeout" in dict_ and dict_["inviteTimeout"] is not None:
            instance.invite_timeout = int(dict_["inviteTimeout"])
        elif include_empty:
            instance.invite_timeout = 0
        if "joinability" in dict_ and dict_["joinability"] is not None:
            instance.joinability = str(dict_["joinability"])
        elif include_empty:
            instance.joinability = ""
        if "last" in dict_ and dict_["last"] is not None:
            instance.last = str(dict_["last"])
        elif include_empty:
            instance.last = ""
        if "maxPlayers" in dict_ and dict_["maxPlayers"] is not None:
            instance.max_players = int(dict_["maxPlayers"])
        elif include_empty:
            instance.max_players = 0
        if "minPlayers" in dict_ and dict_["minPlayers"] is not None:
            instance.min_players = int(dict_["minPlayers"])
        elif include_empty:
            instance.min_players = 0
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "persistent" in dict_ and dict_["persistent"] is not None:
            instance.persistent = bool(dict_["persistent"])
        elif include_empty:
            instance.persistent = False
        if "textChat" in dict_ and dict_["textChat"] is not None:
            instance.text_chat = bool(dict_["textChat"])
        elif include_empty:
            instance.text_chat = False
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "appName" in dict_ and dict_["appName"] is not None:
            instance.app_name = str(dict_["appName"])
        elif include_empty:
            instance.app_name = ""
        if "attributes" in dict_ and dict_["attributes"] is not None:
            instance.attributes = {
                str(k0): v0 for k0, v0 in dict_["attributes"].items()
            }
        elif include_empty:
            instance.attributes = {}
        if "autoJoin" in dict_ and dict_["autoJoin"] is not None:
            instance.auto_join = bool(dict_["autoJoin"])
        elif include_empty:
            instance.auto_join = False
        if "autoLeaveSession" in dict_ and dict_["autoLeaveSession"] is not None:
            instance.auto_leave_session = bool(dict_["autoLeaveSession"])
        elif include_empty:
            instance.auto_leave_session = False
        if "customURLGRPC" in dict_ and dict_["customURLGRPC"] is not None:
            instance.custom_urlgrpc = str(dict_["customURLGRPC"])
        elif include_empty:
            instance.custom_urlgrpc = ""
        if (
            "disableCodeGeneration" in dict_
            and dict_["disableCodeGeneration"] is not None
        ):
            instance.disable_code_generation = bool(dict_["disableCodeGeneration"])
        elif include_empty:
            instance.disable_code_generation = False
        if "disableResendInvite" in dict_ and dict_["disableResendInvite"] is not None:
            instance.disable_resend_invite = bool(dict_["disableResendInvite"])
        elif include_empty:
            instance.disable_resend_invite = False
        if "dsManualSetReady" in dict_ and dict_["dsManualSetReady"] is not None:
            instance.ds_manual_set_ready = bool(dict_["dsManualSetReady"])
        elif include_empty:
            instance.ds_manual_set_ready = False
        if "dsSource" in dict_ and dict_["dsSource"] is not None:
            instance.ds_source = str(dict_["dsSource"])
        elif include_empty:
            instance.ds_source = ""
        if "enableSecret" in dict_ and dict_["enableSecret"] is not None:
            instance.enable_secret = bool(dict_["enableSecret"])
        elif include_empty:
            instance.enable_secret = False
        if "fallbackClaimKeys" in dict_ and dict_["fallbackClaimKeys"] is not None:
            instance.fallback_claim_keys = [
                str(i0) for i0 in dict_["fallbackClaimKeys"]
            ]
        elif include_empty:
            instance.fallback_claim_keys = []
        if "immutableStorage" in dict_ and dict_["immutableStorage"] is not None:
            instance.immutable_storage = bool(dict_["immutableStorage"])
        elif include_empty:
            instance.immutable_storage = False
        if (
            "leaderElectionGracePeriod" in dict_
            and dict_["leaderElectionGracePeriod"] is not None
        ):
            instance.leader_election_grace_period = int(
                dict_["leaderElectionGracePeriod"]
            )
        elif include_empty:
            instance.leader_election_grace_period = 0
        if "manualRejoin" in dict_ and dict_["manualRejoin"] is not None:
            instance.manual_rejoin = bool(dict_["manualRejoin"])
        elif include_empty:
            instance.manual_rejoin = False
        if "maxActiveSessions" in dict_ and dict_["maxActiveSessions"] is not None:
            instance.max_active_sessions = int(dict_["maxActiveSessions"])
        elif include_empty:
            instance.max_active_sessions = 0
        if (
            "nativeSessionSetting" in dict_
            and dict_["nativeSessionSetting"] is not None
        ):
            instance.native_session_setting = (
                ModelsNativeSessionSetting.create_from_dict(
                    dict_["nativeSessionSetting"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.native_session_setting = ModelsNativeSessionSetting()
        if "preferredClaimKeys" in dict_ and dict_["preferredClaimKeys"] is not None:
            instance.preferred_claim_keys = [
                str(i0) for i0 in dict_["preferredClaimKeys"]
            ]
        elif include_empty:
            instance.preferred_claim_keys = []
        if "PSNBaseUrl" in dict_ and dict_["PSNBaseUrl"] is not None:
            instance.psn_base_url = str(dict_["PSNBaseUrl"])
        elif include_empty:
            instance.psn_base_url = ""
        if "requestedRegions" in dict_ and dict_["requestedRegions"] is not None:
            instance.requested_regions = [str(i0) for i0 in dict_["requestedRegions"]]
        elif include_empty:
            instance.requested_regions = []
        if (
            "tieTeamsSessionLifetime" in dict_
            and dict_["tieTeamsSessionLifetime"] is not None
        ):
            instance.tie_teams_session_lifetime = bool(dict_["tieTeamsSessionLifetime"])
        elif include_empty:
            instance.tie_teams_session_lifetime = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsConfigurationTemplateResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsConfigurationTemplateResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsConfigurationTemplateResponse,
        List[ApimodelsConfigurationTemplateResponse],
        Dict[Any, ApimodelsConfigurationTemplateResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "clientVersion": "client_version",
            "createdAt": "created_at",
            "deployment": "deployment",
            "inactiveTimeout": "inactive_timeout",
            "inviteTimeout": "invite_timeout",
            "joinability": "joinability",
            "last": "last",
            "maxPlayers": "max_players",
            "minPlayers": "min_players",
            "name": "name",
            "namespace": "namespace",
            "persistent": "persistent",
            "textChat": "text_chat",
            "type": "type_",
            "updatedAt": "updated_at",
            "appName": "app_name",
            "attributes": "attributes",
            "autoJoin": "auto_join",
            "autoLeaveSession": "auto_leave_session",
            "customURLGRPC": "custom_urlgrpc",
            "disableCodeGeneration": "disable_code_generation",
            "disableResendInvite": "disable_resend_invite",
            "dsManualSetReady": "ds_manual_set_ready",
            "dsSource": "ds_source",
            "enableSecret": "enable_secret",
            "fallbackClaimKeys": "fallback_claim_keys",
            "immutableStorage": "immutable_storage",
            "leaderElectionGracePeriod": "leader_election_grace_period",
            "manualRejoin": "manual_rejoin",
            "maxActiveSessions": "max_active_sessions",
            "nativeSessionSetting": "native_session_setting",
            "preferredClaimKeys": "preferred_claim_keys",
            "PSNBaseUrl": "psn_base_url",
            "requestedRegions": "requested_regions",
            "tieTeamsSessionLifetime": "tie_teams_session_lifetime",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "clientVersion": True,
            "createdAt": True,
            "deployment": True,
            "inactiveTimeout": True,
            "inviteTimeout": True,
            "joinability": True,
            "last": True,
            "maxPlayers": True,
            "minPlayers": True,
            "name": True,
            "namespace": True,
            "persistent": True,
            "textChat": True,
            "type": True,
            "updatedAt": True,
            "appName": False,
            "attributes": False,
            "autoJoin": False,
            "autoLeaveSession": False,
            "customURLGRPC": False,
            "disableCodeGeneration": False,
            "disableResendInvite": False,
            "dsManualSetReady": False,
            "dsSource": False,
            "enableSecret": False,
            "fallbackClaimKeys": False,
            "immutableStorage": False,
            "leaderElectionGracePeriod": False,
            "manualRejoin": False,
            "maxActiveSessions": False,
            "nativeSessionSetting": False,
            "preferredClaimKeys": False,
            "PSNBaseUrl": False,
            "requestedRegions": False,
            "tieTeamsSessionLifetime": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session History Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.apimodels_ticket_observability_history import (
    ApimodelsTicketObservabilityHistory,
)
from ..models.models_ticket import ModelsTicket
from ..models.models_ticket_observability import ModelsTicketObservability


class ApimodelsTicketObservabilityDetail(Model):
    """Apimodels ticket observability detail (apimodels.TicketObservabilityDetail)

    Properties:
        action: (action) REQUIRED str

        created_at: (createdAt) REQUIRED str

        expired_at: (expiredAt) REQUIRED str

        game_mode: (game_mode) REQUIRED str

        match_id: (matchId) REQUIRED str

        namespace: (namespace) REQUIRED str

        party_id: (party_id) REQUIRED str

        regions: (regions) REQUIRED str

        remaining_tickets: (remainingTickets) REQUIRED int

        ticket_id: (ticket_id) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        history: (history) OPTIONAL List[ApimodelsTicketObservabilityHistory]

        last_data: (lastData) OPTIONAL ModelsTicketObservability

        remaining_players_per_ticket: (remainingPlayersPerTicket) OPTIONAL List[int]

        requested_region: (requestedRegion) OPTIONAL List[str]

        ticket: (ticket) OPTIONAL ModelsTicket

        time_to_match_sec: (timeToMatchSec) OPTIONAL float

        user_i_ds: (userIDs) OPTIONAL List[str]
    """

    # region fields

    action: str  # REQUIRED
    created_at: str  # REQUIRED
    expired_at: str  # REQUIRED
    game_mode: str  # REQUIRED
    match_id: str  # REQUIRED
    namespace: str  # REQUIRED
    party_id: str  # REQUIRED
    regions: str  # REQUIRED
    remaining_tickets: int  # REQUIRED
    ticket_id: str  # REQUIRED
    updated_at: str  # REQUIRED
    history: List[ApimodelsTicketObservabilityHistory]  # OPTIONAL
    last_data: ModelsTicketObservability  # OPTIONAL
    remaining_players_per_ticket: List[int]  # OPTIONAL
    requested_region: List[str]  # OPTIONAL
    ticket: ModelsTicket  # OPTIONAL
    time_to_match_sec: float  # OPTIONAL
    user_i_ds: List[str]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_action(self, value: str) -> ApimodelsTicketObservabilityDetail:
        self.action = value
        return self

    def with_created_at(self, value: str) -> ApimodelsTicketObservabilityDetail:
        self.created_at = value
        return self

    def with_expired_at(self, value: str) -> ApimodelsTicketObservabilityDetail:
        self.expired_at = value
        return self

    def with_game_mode(self, value: str) -> ApimodelsTicketObservabilityDetail:
        self.game_mode = value
        return self

    def with_match_id(self, value: str) -> ApimodelsTicketObservabilityDetail:
        self.match_id = value
        return self

    def with_namespace(self, value: str) -> ApimodelsTicketObservabilityDetail:
        self.namespace = value
        return self

    def with_party_id(self, value: str) -> ApimodelsTicketObservabilityDetail:
        self.party_id = value
        return self

    def with_regions(self, value: str) -> ApimodelsTicketObservabilityDetail:
        self.regions = value
        return self

    def with_remaining_tickets(self, value: int) -> ApimodelsTicketObservabilityDetail:
        self.remaining_tickets = value
        return self

    def with_ticket_id(self, value: str) -> ApimodelsTicketObservabilityDetail:
        self.ticket_id = value
        return self

    def with_updated_at(self, value: str) -> ApimodelsTicketObservabilityDetail:
        self.updated_at = value
        return self

    def with_history(
        self, value: List[ApimodelsTicketObservabilityHistory]
    ) -> ApimodelsTicketObservabilityDetail:
        self.history = value
        return self

    def with_last_data(
        self, value: ModelsTicketObservability
    ) -> ApimodelsTicketObservabilityDetail:
        self.last_data = value
        return self

    def with_remaining_players_per_ticket(
        self, value: List[int]
    ) -> ApimodelsTicketObservabilityDetail:
        self.remaining_players_per_ticket = value
        return self

    def with_requested_region(
        self, value: List[str]
    ) -> ApimodelsTicketObservabilityDetail:
        self.requested_region = value
        return self

    def with_ticket(self, value: ModelsTicket) -> ApimodelsTicketObservabilityDetail:
        self.ticket = value
        return self

    def with_time_to_match_sec(
        self, value: float
    ) -> ApimodelsTicketObservabilityDetail:
        self.time_to_match_sec = value
        return self

    def with_user_i_ds(self, value: List[str]) -> ApimodelsTicketObservabilityDetail:
        self.user_i_ds = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "action"):
            result["action"] = str(self.action)
        elif include_empty:
            result["action"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "expired_at"):
            result["expiredAt"] = str(self.expired_at)
        elif include_empty:
            result["expiredAt"] = ""
        if hasattr(self, "game_mode"):
            result["game_mode"] = str(self.game_mode)
        elif include_empty:
            result["game_mode"] = ""
        if hasattr(self, "match_id"):
            result["matchId"] = str(self.match_id)
        elif include_empty:
            result["matchId"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "party_id"):
            result["party_id"] = str(self.party_id)
        elif include_empty:
            result["party_id"] = ""
        if hasattr(self, "regions"):
            result["regions"] = str(self.regions)
        elif include_empty:
            result["regions"] = ""
        if hasattr(self, "remaining_tickets"):
            result["remainingTickets"] = int(self.remaining_tickets)
        elif include_empty:
            result["remainingTickets"] = 0
        if hasattr(self, "ticket_id"):
            result["ticket_id"] = str(self.ticket_id)
        elif include_empty:
            result["ticket_id"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "history"):
            result["history"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.history
            ]
        elif include_empty:
            result["history"] = []
        if hasattr(self, "last_data"):
            result["lastData"] = self.last_data.to_dict(include_empty=include_empty)
        elif include_empty:
            result["lastData"] = ModelsTicketObservability()
        if hasattr(self, "remaining_players_per_ticket"):
            result["remainingPlayersPerTicket"] = [
                int(i0) for i0 in self.remaining_players_per_ticket
            ]
        elif include_empty:
            result["remainingPlayersPerTicket"] = []
        if hasattr(self, "requested_region"):
            result["requestedRegion"] = [str(i0) for i0 in self.requested_region]
        elif include_empty:
            result["requestedRegion"] = []
        if hasattr(self, "ticket"):
            result["ticket"] = self.ticket.to_dict(include_empty=include_empty)
        elif include_empty:
            result["ticket"] = ModelsTicket()
        if hasattr(self, "time_to_match_sec"):
            result["timeToMatchSec"] = float(self.time_to_match_sec)
        elif include_empty:
            result["timeToMatchSec"] = 0.0
        if hasattr(self, "user_i_ds"):
            result["userIDs"] = [str(i0) for i0 in self.user_i_ds]
        elif include_empty:
            result["userIDs"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        action: str,
        created_at: str,
        expired_at: str,
        game_mode: str,
        match_id: str,
        namespace: str,
        party_id: str,
        regions: str,
        remaining_tickets: int,
        ticket_id: str,
        updated_at: str,
        history: Optional[List[ApimodelsTicketObservabilityHistory]] = None,
        last_data: Optional[ModelsTicketObservability] = None,
        remaining_players_per_ticket: Optional[List[int]] = None,
        requested_region: Optional[List[str]] = None,
        ticket: Optional[ModelsTicket] = None,
        time_to_match_sec: Optional[float] = None,
        user_i_ds: Optional[List[str]] = None,
        **kwargs,
    ) -> ApimodelsTicketObservabilityDetail:
        instance = cls()
        instance.action = action
        instance.created_at = created_at
        instance.expired_at = expired_at
        instance.game_mode = game_mode
        instance.match_id = match_id
        instance.namespace = namespace
        instance.party_id = party_id
        instance.regions = regions
        instance.remaining_tickets = remaining_tickets
        instance.ticket_id = ticket_id
        instance.updated_at = updated_at
        if history is not None:
            instance.history = history
        if last_data is not None:
            instance.last_data = last_data
        if remaining_players_per_ticket is not None:
            instance.remaining_players_per_ticket = remaining_players_per_ticket
        if requested_region is not None:
            instance.requested_region = requested_region
        if ticket is not None:
            instance.ticket = ticket
        if time_to_match_sec is not None:
            instance.time_to_match_sec = time_to_match_sec
        if user_i_ds is not None:
            instance.user_i_ds = user_i_ds
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsTicketObservabilityDetail:
        instance = cls()
        if not dict_:
            return instance
        if "action" in dict_ and dict_["action"] is not None:
            instance.action = str(dict_["action"])
        elif include_empty:
            instance.action = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "expiredAt" in dict_ and dict_["expiredAt"] is not None:
            instance.expired_at = str(dict_["expiredAt"])
        elif include_empty:
            instance.expired_at = ""
        if "game_mode" in dict_ and dict_["game_mode"] is not None:
            instance.game_mode = str(dict_["game_mode"])
        elif include_empty:
            instance.game_mode = ""
        if "matchId" in dict_ and dict_["matchId"] is not None:
            instance.match_id = str(dict_["matchId"])
        elif include_empty:
            instance.match_id = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "party_id" in dict_ and dict_["party_id"] is not None:
            instance.party_id = str(dict_["party_id"])
        elif include_empty:
            instance.party_id = ""
        if "regions" in dict_ and dict_["regions"] is not None:
            instance.regions = str(dict_["regions"])
        elif include_empty:
            instance.regions = ""
        if "remainingTickets" in dict_ and dict_["remainingTickets"] is not None:
            instance.remaining_tickets = int(dict_["remainingTickets"])
        elif include_empty:
            instance.remaining_tickets = 0
        if "ticket_id" in dict_ and dict_["ticket_id"] is not None:
            instance.ticket_id = str(dict_["ticket_id"])
        elif include_empty:
            instance.ticket_id = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "history" in dict_ and dict_["history"] is not None:
            instance.history = [
                ApimodelsTicketObservabilityHistory.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["history"]
            ]
        elif include_empty:
            instance.history = []
        if "lastData" in dict_ and dict_["lastData"] is not None:
            instance.last_data = ModelsTicketObservability.create_from_dict(
                dict_["lastData"], include_empty=include_empty
            )
        elif include_empty:
            instance.last_data = ModelsTicketObservability()
        if (
            "remainingPlayersPerTicket" in dict_
            and dict_["remainingPlayersPerTicket"] is not None
        ):
            instance.remaining_players_per_ticket = [
                int(i0) for i0 in dict_["remainingPlayersPerTicket"]
            ]
        elif include_empty:
            instance.remaining_players_per_ticket = []
        if "requestedRegion" in dict_ and dict_["requestedRegion"] is not None:
            instance.requested_region = [str(i0) for i0 in dict_["requestedRegion"]]
        elif include_empty:
            instance.requested_region = []
        if "ticket" in dict_ and dict_["ticket"] is not None:
            instance.ticket = ModelsTicket.create_from_dict(
                dict_["ticket"], include_empty=include_empty
            )
        elif include_empty:
            instance.ticket = ModelsTicket()
        if "timeToMatchSec" in dict_ and dict_["timeToMatchSec"] is not None:
            instance.time_to_match_sec = float(dict_["timeToMatchSec"])
        elif include_empty:
            instance.time_to_match_sec = 0.0
        if "userIDs" in dict_ and dict_["userIDs"] is not None:
            instance.user_i_ds = [str(i0) for i0 in dict_["userIDs"]]
        elif include_empty:
            instance.user_i_ds = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsTicketObservabilityDetail]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsTicketObservabilityDetail]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsTicketObservabilityDetail,
        List[ApimodelsTicketObservabilityDetail],
        Dict[Any, ApimodelsTicketObservabilityDetail],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "action": "action",
            "createdAt": "created_at",
            "expiredAt": "expired_at",
            "game_mode": "game_mode",
            "matchId": "match_id",
            "namespace": "namespace",
            "party_id": "party_id",
            "regions": "regions",
            "remainingTickets": "remaining_tickets",
            "ticket_id": "ticket_id",
            "updatedAt": "updated_at",
            "history": "history",
            "lastData": "last_data",
            "remainingPlayersPerTicket": "remaining_players_per_ticket",
            "requestedRegion": "requested_region",
            "ticket": "ticket",
            "timeToMatchSec": "time_to_match_sec",
            "userIDs": "user_i_ds",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "action": True,
            "createdAt": True,
            "expiredAt": True,
            "game_mode": True,
            "matchId": True,
            "namespace": True,
            "party_id": True,
            "regions": True,
            "remainingTickets": True,
            "ticket_id": True,
            "updatedAt": True,
            "history": False,
            "lastData": False,
            "remainingPlayersPerTicket": False,
            "requestedRegion": False,
            "ticket": False,
            "timeToMatchSec": False,
            "userIDs": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Achievement Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_psn_event_data import ModelsPSNEventData


class ModelsPSNEvent(Model):
    """Models PSN event (models.PSNEvent)

    Properties:
        date_time: (dateTime) REQUIRED str

        event_data: (eventData) REQUIRED ModelsPSNEventData

        event_id: (eventId) REQUIRED str

        version: (version) REQUIRED str
    """

    # region fields

    date_time: str  # REQUIRED
    event_data: ModelsPSNEventData  # REQUIRED
    event_id: str  # REQUIRED
    version: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_date_time(self, value: str) -> ModelsPSNEvent:
        self.date_time = value
        return self

    def with_event_data(self, value: ModelsPSNEventData) -> ModelsPSNEvent:
        self.event_data = value
        return self

    def with_event_id(self, value: str) -> ModelsPSNEvent:
        self.event_id = value
        return self

    def with_version(self, value: str) -> ModelsPSNEvent:
        self.version = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "date_time"):
            result["dateTime"] = str(self.date_time)
        elif include_empty:
            result["dateTime"] = ""
        if hasattr(self, "event_data"):
            result["eventData"] = self.event_data.to_dict(include_empty=include_empty)
        elif include_empty:
            result["eventData"] = ModelsPSNEventData()
        if hasattr(self, "event_id"):
            result["eventId"] = str(self.event_id)
        elif include_empty:
            result["eventId"] = ""
        if hasattr(self, "version"):
            result["version"] = str(self.version)
        elif include_empty:
            result["version"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        date_time: str,
        event_data: ModelsPSNEventData,
        event_id: str,
        version: str,
        **kwargs,
    ) -> ModelsPSNEvent:
        instance = cls()
        instance.date_time = date_time
        instance.event_data = event_data
        instance.event_id = event_id
        instance.version = version
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsPSNEvent:
        instance = cls()
        if not dict_:
            return instance
        if "dateTime" in dict_ and dict_["dateTime"] is not None:
            instance.date_time = str(dict_["dateTime"])
        elif include_empty:
            instance.date_time = ""
        if "eventData" in dict_ and dict_["eventData"] is not None:
            instance.event_data = ModelsPSNEventData.create_from_dict(
                dict_["eventData"], include_empty=include_empty
            )
        elif include_empty:
            instance.event_data = ModelsPSNEventData()
        if "eventId" in dict_ and dict_["eventId"] is not None:
            instance.event_id = str(dict_["eventId"])
        elif include_empty:
            instance.event_id = ""
        if "version" in dict_ and dict_["version"] is not None:
            instance.version = str(dict_["version"])
        elif include_empty:
            instance.version = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsPSNEvent]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsPSNEvent]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ModelsPSNEvent, List[ModelsPSNEvent], Dict[Any, ModelsPSNEvent]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "dateTime": "date_time",
            "eventData": "event_data",
            "eventId": "event_id",
            "version": "version",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "dateTime": True,
            "eventData": True,
            "eventId": True,
            "version": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Event Log Service."""

__version__ = "2.2.5"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .get_event_by_event_id_handler import GetEventByEventIDHandler
from .get_event_by_event_type_8d579c import GetEventByEventTypeAndEventIDHandler
from .get_event_by_event_type_f075bc import GetEventByEventTypeHandler
from .get_event_by_namespace_handler import GetEventByNamespaceHandler
from .get_event_by_user_event_af55e0 import GetEventByUserEventIDAndEventTypeHandler
from .get_event_by_user_id_an_30e3ad import GetEventByUserIDAndEventIDHandler
from .get_event_by_user_id_an_c0de1a import GetEventByUserIDAndEventTypeHandler
from .get_event_by_user_id_handler import GetEventByUserIDHandler
from .post_event_handler import PostEventHandler



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Ds Artifact Manager."""

__version__ = "1.11.5"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from ._all_terminated_servers import list_terminated_servers
from ._all_terminated_servers import list_terminated_servers_async

from ._artifact_upload_process_queue import delete_active_queue
from ._artifact_upload_process_queue import delete_active_queue_async
from ._artifact_upload_process_queue import delete_node_by_id
from ._artifact_upload_process_queue import delete_node_by_id_async
from ._artifact_upload_process_queue import delete_queue
from ._artifact_upload_process_queue import delete_queue_async
from ._artifact_upload_process_queue import get_active_queue
from ._artifact_upload_process_queue import get_active_queue_async
from ._artifact_upload_process_queue import list_all_active_queue
from ._artifact_upload_process_queue import list_all_active_queue_async
from ._artifact_upload_process_queue import list_all_queue
from ._artifact_upload_process_queue import list_all_queue_async
from ._artifact_upload_process_queue import list_nodes_ip_address
from ._artifact_upload_process_queue import list_nodes_ip_address_async
from ._artifact_upload_process_queue import list_queue
from ._artifact_upload_process_queue import list_queue_async
from ._artifact_upload_process_queue import report_failed_upload
from ._artifact_upload_process_queue import report_failed_upload_async
from ._artifact_upload_process_queue import set_active_queue
from ._artifact_upload_process_queue import set_active_queue_async

from ._download_server_artifact import check_server_artifact
from ._download_server_artifact import check_server_artifact_async
from ._download_server_artifact import download_server_artifacts
from ._download_server_artifact import download_server_artifacts_async

from ._operations import public_get_messages
from ._operations import public_get_messages_async

from ._terminated_servers import list_terminated_servers_with_namespace
from ._terminated_servers import list_terminated_servers_with_namespace_async
